//
// Created by robin on 1/7/18.
//

#ifndef QDNSAGG_QDNSHEADER_H
#define QDNSAGG_QDNSHEADER_H

#include <qplatformdefs.h>
#include <QByteArray>
#include <QDataStream>
#include <QDebug>
#include "QDnsBase.h"

namespace QDns
{
    namespace Base
    {
        class QDnsHeader
        {
        public:
            QDnsHeader();

            explicit QDnsHeader(const QByteArray &data);

            explicit QDnsHeader(QDataStream &dataStream);

            virtual ~QDnsHeader() = default;

            QByteArray serialize() const;

            void serialize(QDataStream &dataStream) const;

            bool isValid() const;

            quint16 getId() const;

            void setId(quint16 id);


            quint16 getFlags() const;

            void setFlags(quint16 flags);

            void setFlagBits(quint8 bit, quint16 value, quint8 bitCount);

            quint16 getFlagBits(quint8 bit, quint8 bitCount) const;

            static quint8 getFlagBitIndex(quint8 bit);

            bool isQuery() const;

            void setIsQuery(bool isQuery);

            QDnsBase::HeaderOpcode getOpcode() const;

            void setOpcode(QDnsBase::HeaderOpcode opcode);

            bool isAuthoritative() const;

            void setIsAuthoritative(bool isAuthoritative);

            bool isTruncated() const;

            void setIsTruncated(bool isTruncated);

            bool isRecursionDesired() const;

            void setIsRecursionDesired(bool isRecursionDesired);

            bool isRecursionAvailable() const;

            void setIsRecursionAvailable(bool isRecursionAvailable);

            bool isAnswerAuthenticated() const;

            void setIsAnswerAuthenticated(bool isAnswerAuthenticated);

            bool isNonAuthenticatedAcceptable() const;

            void setNonAuthenticatedAcceptable(bool isNonAuthenticatedAcceptable);

            QDnsBase::HeaderReplyCode getReplyCode() const;

            void setReplyCode(QDnsBase::HeaderReplyCode replyCode);


            quint16 getQueryCount() const;

            void setQueryCount(quint16 questionCount);

            quint16 getAnswerCount() const;

            void setAnswerCount(quint16 answerCount);

            quint16 getAuthorityCount() const;

            void setAuthorityCount(quint16 authorityCount);

            quint16 getAdditionalCount() const;

            void setAdditionalCount(quint16 additionalCount);

            QString toString() const;

        private:
            void parse(QDataStream &dataStream);

            bool m_isValid;
            quint16 m_id;
            quint16 m_flags;
            quint16 m_queryCount;
            quint16 m_answerCount;
            quint16 m_authorityCount;
            quint16 m_additionalCount;
        };

        QDebug operator<<(QDebug debug, const QDnsHeader &h);
    }
}

#endif //QDNSAGG_QDNSHEADER_H
