//
// Created by robin on 1/23/18.
//

#include "dns-base/QDnsLabel.h"
#include "dns-base/RecordData/QDnsRecordDataMX.h"

namespace QDns
{
    namespace Base
    {
        QDnsRecordDataMX::QDnsRecordDataMX()
                : QDnsRecordData()
        {
        }

        QDnsRecordDataMX::QDnsRecordDataMX(const QByteArray &packetData, quint32 pos)
                : QDnsRecordDataMX()
        {
            QDataStream dataStream(packetData);
            dataStream.skipRawData(pos);
            parse(packetData, dataStream);
        }

        QDnsRecordDataMX::QDnsRecordDataMX(const QByteArray &packetData, QDataStream &dataStream)
                : QDnsRecordDataMX()
        {
            parse(packetData, dataStream);
        }

        void QDnsRecordDataMX::parse(const QByteArray &packetData, QDataStream &dataStream)
        {
            if (dataStream.device()->bytesAvailable() < 3)
            {
                m_isValid = false;
            }
            else
            {
                dataStream >> m_preference;
                m_mx = QDnsLabel::parse(packetData, dataStream);
                if (m_mx.isNull())
                {
                    m_isValid = false;
                }
            }
        }

        QByteArray QDnsRecordDataMX::serialize() const
        {
            QByteArray byteArray;
            QDataStream dataStream(&byteArray, QIODevice::WriteOnly);
            serialize(dataStream);
            return byteArray;
        }

        void QDnsRecordDataMX::serialize(QDataStream &dataStream) const
        {
            dataStream << m_preference;
            QDnsLabel::serialize(m_mx, dataStream);
        }

        QDnsRecordData *QDnsRecordDataMX::clone() const
        {
            auto data = new QDnsRecordDataMX();
            data->m_isValid = m_isValid;
            data->m_preference = m_preference;
            data->m_mx = m_mx;
            return data;
        }

        quint16 QDnsRecordDataMX::getPreference() const
        {
            return m_preference;
        }

        void QDnsRecordDataMX::setPreference(quint16 preference)
        {
            m_preference = preference;
        }

        const QString &QDnsRecordDataMX::getMx() const
        {
            return m_mx;
        }

        void QDnsRecordDataMX::setMx(const QString &mx)
        {
            m_mx = mx;
        }

        QString QDnsRecordDataMX::toString() const
        {
            return QString::number(getPreference()) + " " + getMx();
        }
    }
}